import os
import requests


def get_token_and_base_url(client, salesforce_test_instance=False):
    params = {
        "grant_type": "password",
        "client_id": os.environ["SALESFORCE_%s_CLIENT_ID" % client],
        "client_secret": os.environ["SALESFORCE_%s_CLIENT_SECRET" % client],
        "username": os.environ["SALESFORCE_%s_USERNAME" % client],
        "password": os.environ["SALESFORCE_%s_PASSWORD" % client] + os.environ["SALESFORCE_%s_SECURITY_TOKEN" % client],
    }
    if not salesforce_test_instance:
        url = "https://login.salesforce.com/services/oauth2/token"
    else:
        url = "https://test.salesforce.com/services/oauth2/token"

    r = requests.post(url, params=params).json()
    if r.get("error"):
        return r
    else:
        return r.get("access_token"), r.get('instance_url')



#
with open('orders_test_4.csv', 'w', encoding='utf8', newline='\n') as output_file:
    fc = csv.DictWriter(output_file, fieldnames=columns)
    fc.writeheader()
    fc.writerows(data)
with open('orders_test_4.csv', 'r') as f:
    fc = f.read()

find_str = 'False'
replace_str = '0'
new_csv_str = re.sub(find_str, replace_str, fc)

find_str = 'True'
replace_str = '1'

new_csv_str_1 = re.sub(find_str, replace_str, new_csv_str)

with open('orders_test_5.csv', 'w', encoding='utf8', newline='\n') as f:
    f.write(new_csv_str_1)

#
#
#

#
# df = pd.read_fwf('orders_test_4.csv',delimiter=',')
# df.to_csv('orders_test_5.csv')
# substitute

time2 = time.time()
# print(time2-time1)
connect_str = os.getenv('AZURE_STORAGE_CONNECTION_STRING')
blob_service_client = BlobServiceClient.from_connection_string(connect_str)
# container_client = blob_service_client.create_container('orders')
blob_client = blob_service_client.get_blob_client(container='orders', blob='orders_test_6.csv')
print("\nUploading to Azure Storage as blob:\n\t" + 'orders_f.csv')
# Upload the created file
with open('orders_test_5.csv', "rb") as data_cc:
    blob_client.upload_blob(data_cc)
#
# time3=time.time()
# print(time3-time2)
# insert_query = '''BULK INSERT %s.%s FROM "%s" WITH (DATA_SOURCE = '%s', FIELDTERMINATOR = ',', ROWTERMINATOR = '\n',FIRSTROW =2);''' % ('chandon_salesforce', 'order_1','chandon_orders.csv','MHUSBlob')
# print(insert_query)
# self.dbstream.execute_query(insert_query)
# time4=time.time()


import csv
import os

from azure.storage.blob import BlobServiceClient

# with open('people.csv', 'w', encoding='utf8', newline='') as output_file:
#     fc = csv.DictWriter(output_file, fieldnames=columns)
#     fc.writeheader()
#     fc.writerows(data)




def to_blob():
    connect_str = os.getenv('AZURE_STORAGE_CONNECTION_STRING')
    blob_service_client = BlobServiceClient.from_connection_string(connect_str)
    container_name = "orders"
    print(container_name)
    container_client = blob_service_client.create_container(container_name)
    local_file_name = "order_testing5.csv"
    upload_file_path = "order_testing5.csv"
    blob_client = blob_service_client.get_blob_client(container=container_name, blob=upload_file_path)
    print("\nUploading to Azure Storage as blob:\n\t" + local_file_name)
    # Upload the created file
    with open(upload_file_path, "rb") as data:
        blob_client.upload_blob(data)



def process_data(raw_data):
    object_row = []
    for r in raw_data:
        _object = dict()
        for o in r.keys():
            if type(r.get(o)) == dict:
                _object[o.lower()] = str(r.get(o))
            else:
                _object[o.lower()] = r.get(o)
        object_row.append(_object)
    return object_row


def get_column_names(data):
    column_list = []
    for d in data:
        for c in d.keys():
            if c not in column_list:
                column_list.append(c)
    print(len(column_list))
    return column_list


def send_temp_data(datamart, data, schema_prefix, table, column_names):
    data_to_send = {
        "columns_name": column_names,
        "rows": [[r[c] for c in column_names] for r in data],
        "table_name": schema_prefix + '.' + table + '_temp'}
    datamart.send_data(
        data=data_to_send,
        other_table_to_update=schema_prefix + '.' + table,
        replace=False)


def _clean(datamart, schema_prefix, table):
    selecting_id = 'id'
    print('trying to clean')
    cleaning_query = """
            DELETE FROM %(schema_name)s.%(table_name)s WHERE %(id)s IN (SELECT distinct %(id)s FROM %(schema_name)s.%(table_name)s_temp);
            INSERT INTO %(schema_name)s.%(table_name)s 
            SELECT * FROM %(schema_name)s.%(table_name)s_temp;
            DELETE FROM %(schema_name)s.%(table_name)s_temp;
            """ % {"table_name": table,
                   "schema_name": schema_prefix,
                   "id": selecting_id}
    datamart.execute_query(cleaning_query)
    print('cleaned')



import yaml
import requests
from pysalesforce.auth import get_token_and_base_url
from pysalesforce.useful import process_data, get_column_names, _clean, send_temp_data


class Salesforce:
    # >>>Si on fait un call avec la mauvaise version et traiter proprement l'erreur pour que ce soit forcément explicite

    def __init__(self, var_env_key, dbstream, config_file_path, salesforce_test_instance=False, api_version=None):
        self.var_env_key = var_env_key
        self.dbstream = dbstream
        self.config_file_path = config_file_path
        self.salesforce_test_instance = salesforce_test_instance
        self.access_token, self.base_url = get_token_and_base_url(var_env_key, self.salesforce_test_instance)
        self.api_version = api_version
        self._objects = yaml.load(open(self.config_file_path), Loader=yaml.FullLoader).get('objects')
        self.schema_prefix = yaml.load(open(self.config_file_path), Loader=yaml.FullLoader).get("schema_prefix")

    def get_endpoint(self):
        config = yaml.load(open(self.config_file_path), Loader=yaml.FullLoader)
        return config.get("endpoints")

    def get_table(self, _object_key):
        _object = self._objects[_object_key]
        if not _object.get('table'):
            return _object_key.lower() + 's'
        return _object.get('table')

    def describe_objects(self, object_name):
        headers = {
            "Authorization": "Bearer %s" % self.access_token
        }
        url = self.base_url + "/services/data/%s/sobjects/%s/describe/" % (self.api_version, object_name)
        result = requests.get(url, headers=headers).json()
        return [r["name"] for r in result["fields"]]

    def query(self, object_name, since):
        fields = self.describe_objects(object_name)
        where_clause = ""
        if since:
            if 'LastModifiedDate' in fields:
                where_clause = " where lastmodifieddate >= %s" % since
        query = 'select '
        for p in fields:
            query += p + ','
        query = query[:-1]
        query += ' from ' + object_name + where_clause
        return query

    def execute_query(self, _object_key, batch_size, since, next_records_url=None):
        result = []
        headers = {
            "Authorization": "Bearer %s" % self.access_token,
            'Accept': 'application/json',
            'Content-type': 'application/json'
        }
        params = {
            "q": self.query(_object_key, since)
        }
        url = self.base_url + "/services/data/%s/query/" % self.api_version
        if not next_records_url:
            r = requests.get(url, params=params, headers=headers).json()
        else:
            r = requests.get(self.base_url + next_records_url, headers=headers).json()
        result = result + r.get("records")
        next_records_url = r.get('nextRecordsUrl')
        i = 1
        while i < batch_size and next_records_url:
            r = requests.get(self.base_url + next_records_url, headers=headers).json()
            result = result + r["records"]
            next_records_url = r.get('nextRecordsUrl')
            i = i + 1
        return {"records": result, "object": _object_key, "next_records_url": r.get('nextRecordsUrl')}

    def retrieve_endpoint(self, endpoint, since=None):
        headers = {
            "Authorization": "Bearer %s" % self.access_token
        }
        params = {}
        if since:
            params = {'lastModificationDate': since}
        url = self.base_url + "/services/apexrest/%s" % endpoint
        r = requests.get(url, headers=headers, params=params).json()
        return r

    def main(self, _object_key, since=None, batchsize=10):
        print('Starting ' + _object_key)

        _object = self._objects[_object_key]
        schema = self.schema_prefix
        table = self.get_table(_object_key)
        next_url = None

        if _object.get("endpoint"):
            raw_data = self.retrieve_endpoint(_object_key, since)
            data = process_data(raw_data)
        else:
            raw_data = self.execute_query(_object_key, batchsize, since)
            next_url = raw_data.get("next_records_url")
            data = process_data(raw_data["records"])

        columns = get_column_names(data)
        send_temp_data(self.dbstream, data, schema, table, columns)

        while next_url:
            raw_data = self.execute_query(_object_key, batchsize, next_records_url=next_url, since=None)
            next_url = raw_data.get("next_records_url")
            data = process_data(raw_data["records"])
            send_temp_data(self.dbstream, data, schema, table, columns)

        print('Ended ' + _object_key)

        _clean(self.dbstream, schema, table)
